<?php

namespace Drupal\xray_audit\Plugin\tasks\QueryGroup;

use Drupal\taxonomy\Entity\Vocabulary;

/**
 * Plugin implementation of queries_data_node.
 *
 * @XrayAuditTaskPlugin (
 *   id = "queries_data_vocabulary",
 *   label = @Translation("Data about vocabulary"),
 *   description = @Translation("Queries execute on database to get reports about vocabularies."),
 *   group = "queries_data",
 *   sort = 3,
 *   operations = {
 *      "vocabulary_description" = {
 *          "label" = "Vocabularies",
 *          "description" = "Vocabulary description."
 *       },
 *       "vocabulary_count" = {
 *          "label" = "Vocabulary count",
 *          "description" = "Vocabulary Count"
 *       },
 *      "vocabulary_count_per_language" = {
 *          "label" = "Vocabulary count per language",
 *          "description" = "Vocabulary count per language."
 *       }
 *    },
 *   dependencies = {"taxonomy"}
 *
 * )
 */
class XrayAuditQueryTaskVocabularyPlugin extends XrayAuditQueryTaskPluginBase {

  /**
   * {@inheritdoc}
   */
  public function getDataOperationResult(string $operation = NULL) {
    switch ($operation) {
      case 'vocabulary_description':
        return $this->vocabulariesDescription();

      case 'vocabulary_count':
        return $this->vocabulariesCount();

      case 'vocabulary_count_per_language':
        return $this->vocabulariesCountPerLanguage();
    }
    return [];
  }

  /**
   * Vocabularies description.
   *
   * @return array
   *   Render array.
   */
  public function vocabulariesDescription() {
    $headerTable = [
      $this->t('id'),
      $this->t('label'),
      $this->t('description'),
      $this->t('langcode')
    ];
    $resultTable = [];
    $vocabularies = Vocabulary::loadMultiple();
    foreach ($vocabularies as $key => $vocabulary) {
      $resultTable[] = [
        $key,
        $vocabulary->label(),
        $vocabulary->get('description'),
        $vocabulary->get('langcode'),
      ];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];
  }

  /**
   * Vocabularies count.
   *
   * @return array
   *   Render array.
   */
  public function vocabulariesCount() {
    $headerTable = [
      $this->t('vid'),
      $this->t('count'),
      $this->t('language')
    ];
    $resultTable = [];
    $alias = 'count';

    $language_by_default = $this->languageManager->getDefaultLanguage()->getId();

    $storage = $this->entityTypeManager->getStorage('taxonomy_term');
    $result = $storage->getAggregateQuery()->groupBy('vid')->aggregate('tid', 'COUNT', $language_by_default, $alias)->execute();
    foreach ($result as $row) {
      if (empty($row['vid'])) {
        continue;
      }
      $resultTable[] = [$row['vid'], $row['count'], $language_by_default];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];
  }

  /**
   * Vocabularies count per language.
   *
   * @return array
   *   Render array.
   */
  public function vocabulariesCountPerLanguage() {
    $headerTable = [
      $this->t('vid'),
      $this->t('langcode'),
      $this->t('count')
    ];
    $resultTable = [];
    $alias = 'count';

    $storage = $this->entityTypeManager->getStorage('taxonomy_term');
    $result = $storage->getAggregateQuery()->groupBy('vid')->groupBy('langcode')->aggregate('tid', 'COUNT', NULL, $alias)->execute();
    foreach ($result as $row) {
      $resultTable[] = [$row['vid'], $row['langcode'], $row['count']];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];
  }

}
